#include "image.h"

template <class T> void Timage<T>::alloc(int _w,int _h)
{
	w=_w;h=_h;
	g=new T[w*h];
	count=new int(1);
}

template <class T>void Timage<T>::kill()
{
	if (!count)
		return;
	(*count)--;
	if (!(*count)) {
		delete[] g;
		delete count;
	}
}

template <class T>void Timage<T>::copy(const Timage<T>& I)
{
	if (!I.count) {
		count=0;
		return;
	}
	count=I.count;
	g=I.g;
	w=I.w;
	h=I.h;
	(*count)++;
}

template <class T> Timage<T>::Timage()
{
	count=0;
}

template <class T> Timage<T>::Timage(int _w,int _h)
{
	alloc(_w,_h);
}

template <class T> Timage<T>::Timage(const Timage<T>& I)
{
	copy(I);
}

template <class T> Timage<T>::~Timage()
{
	kill();
}

template <class T> const Timage<T>& Timage<T>::operator=(const Timage<T>& I)
{
	kill();
	copy(I);
	return I;
}

template <class T> Timage<T> Timage<T>::clone() const
{
	Timage<T> I(w,h);
	for (int i=0;i<w*h;i++)
		I.g[i]=g[i];
	return I;
}
// ===================================================

template Timage<byte>; // Inst. explicite

image::image(const dimage& I,bool rescale)
{
	alloc(I.width(),I.height());
	if (!rescale) 
		for (int i=0;i<w;i++)
			for (int j=0;j<h;j++) {
				if (I(i,j)<0)
					g[i+w*j]=0;
				else if (I(i,j)>255)
					g[i+w*j]=255;
				else
					g[i+w*j]=byte(I(i,j));
			}
	else {
		double m,M;
		I.range(m,M);
		if (m==M)
			M++;
		for (int i=0;i<w;i++)
			for (int j=0;j<h;j++) {
				g[i+w*j]=byte(255*(I(i,j)-m)/(M-m));
			}
	}
}

image::image(const char* name)
{
	count=new int(1);
	LoadGreyImage((char*)name,g,w,h);
}

void image::display(int x,int y) const
{
	PutGreyImage(x,y,g,w,h);
}

//===========================================

#include "rec.h"
template Timage<double>; // Inst. explicite

dimage::dimage(const image& ima)
{
	alloc(ima.width(),ima.height());
	for (int i=0;i<w;i++)
		for (int j=0;j<h;j++)
			g[i+w*j]=ima(i,j);
}

dimage::dimage(const cimage& ima,bool phase)
{
	alloc(ima.width(),ima.height());
	for (int i=0;i<w;i++)
		for (int j=0;j<h;j++)
			g[i+w*j]=phase?arg(ima(i,j)):abs(ima(i,j));
}

void dimage::range(double &m,double &M) const
{
	m=M=g[0];
	for (int i=1;i<w*h;i++) {
		if (m>g[i])
			m=g[i];
		if (M<g[i])
			M=g[i];
	}
}

void dimage::display(int x,int y,bool rescale) const
{
	image ima(*this,rescale);
	ima.display(x,y);
}

dimage dimage::deriche(int dx,int dy,double alpha) const
{
	dimage D(w,h);
	RecDericheFilter(g,w,h,dx,dy,alpha,D.g);
	return D;
}

//============================

#include "fft.h"
template Timage<complexe>; // Inst. explicite

cimage::cimage(const dimage& ima)
{
	alloc(ima.width(),ima.height());
	for (int i=0;i<w;i++)
		for (int j=0;j<h;j++)
			g[i+w*j]=complexe(ima(i,j));
}

void cimage::display(int x,int y,bool phase) const
{
	dimage D(*this,phase);
	D.display(x,y);
}

void cimage::logDisplay(int x,int y) const
{
	dimage D(w,h);
	for (int i=0;i<w;i++)
		for (int j=0;j<h;j++)
			D((i+w/2)%w,(j+h/2)%h)=log(abs((*this)(i,j)));
	D.display(x,y);
}

cimage cimage::fft(bool inverse) const
{
	cimage F=clone();
	// Colonnes
	for (int i=0;i<w;i++)
		Fft(F.g+i,h,inverse,w);
	// Lignes
	for (int j=0;j<h;j++)
		Fft(F.g+j*w,w,inverse,1);
	return F;
}
