#ifndef _lmeds_h
#define _lmeds_h

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * Least median squares technique
 *
 * return 1 if success, 0 if fail
 */
extern int
LMedS(const int nbData,	/* I: the number of Data, which is ordered from 0 */
	  const int sizeSet,	/* I: size of each randomly selected subset of data */
	  int **sets,		/* I: randomly selected subsets, can be set to NULL,
				* then a default routine will be used to generate the subsets */
	  int nbSets,		/* I: the number of subsets, not used if sets == NULL */
	  void (*residual)(const double *x, /* I: parameter vector */
			   int nb, /* I: number of residuals to be computed */
			   const int *indexes, /* I: indexes of data to be used */
			   double *resid /* O: computed residuals */
	      ),		/* I: function which calculates the residuals */
	  int (*estimator)(double *x, /* O: estimated parameter vector */
			   int nb, /* I: number of data to be used */
			   const int *indexes /* I: indexes of data to be used */
	      ),		/* I: function which estimate the parameters
				 * returns the number of solutions */
	  const int isResidualSqr, /* I: set 1 if `residual' computes the squared residuals */
	  const int verbose,	/* I: verbose mode */
	  const int maxNbSol,	/* I: maximum number of solutions given by "estimator" */
	  const double gate,	/* I: gate value for outlier detection, a value in [2, 4] */
	  const double prematureResidual,
				/* I: if > 0.0, LMedS stops when a subset gives a median
				 * residual less than `prematureResidual'.
				 * The specified value must be in accordance with `residual' */
	  const int dim,	/* I: dimension of the parameter vector */
	  const double percentageOfGoodData, /* I: the percentage
	  (between epsilon and 1.0) of good data. An appropriate
	  choice is often 0.5 */
	  double *estimate,	/* O: the corresponding estimate of parameters */
	  int *bestSet,		/* O: best subset retained, can be set to NULL */
	  int *outliers,	/* O: the outliers detected by LMedS */
	  int *nbOutliers	/* O: the number of detected outliers */
    );

#ifdef __cplusplus
}
#endif

#endif 
